@@ -29,13 +29,41 @@ Navier Stokes Finite Differences Solver
 """
 
 import os
+import time
 import argparse
+import pathlib
+from multiprocessing import cpu_count, Pool, current_process
 from fdgrid import mesh
-from nsfds2.init import CfgSetup, Fields
+from nsfds2.init import CfgSetup, create_template
 from nsfds2.fdtd import FDTD
 from nsfds2.utils import files, headers, graphics, sounds
 
 
+class VirtualArguments:
+
+    def __init__(self, cfgfile, command='solve', quiet=True):
+        """Generate a set of virtual arguments as for configparser"""
+        self.cfgfile = cfgfile
+        self.command = command
+        self.quiet = quiet
+
+
+def _wrapper(run, args):
+    """Wrapper function for ploop jobs."""
+    cur = current_process().name
+    ti = time.perf_counter()
+    print(f'{cur} - Solving {args.cfgfile}')
+    run(args)
+    ts = time.perf_counter() - ti
+    print(f'{cur} - {args.cfgfile} solved in {ts:.2f} s')
+    ti = time.perf_counter()
+    cfg = CfgSetup(args=args)
+    plt = graphics.Plot(cfg.datafile, quiet=args.quiet)
+    plt.movie()
+    tm = time.perf_counter() - ti
+    print(f'{cur} - {args.cfgfile} movie made in {tm:.2f} s')
+
+
 def parse_args():
     """ Parse arguments. """
 
@@ -57,6 +85,10 @@ def parse_args():
     data.add_argument('-d', '--dat-file', metavar='DF', dest='datafile',
                       help='path to hdf5 data file')
 
+    path = argparse.ArgumentParser(add_help=False)
+    path.add_argument('-p', dest='path', type=str, required=True,
+                      help='loop over this path')
+
     time = argparse.ArgumentParser(add_help=False)
     time.add_argument('-t', '--timings', action="store_true", default=None,
                       help='Display complete timings')
@@ -68,7 +100,6 @@ def parse_args():
     commands = root.add_subparsers(dest='command',
                                    help='see nsfds2 `command` -h for further help')
 
-
     commands.add_parser("solve", parents=[commons, view, data, time],
                         description="Navier-Stokes equation solver",
                         help="solve NS equations with given configuration")
@@ -78,7 +109,12 @@ def parse_args():
     mak = commands.add_parser("make",
                               description="Make movie/sound files",
                               help="make movie/sound files")
-
+    loop = commands.add_parser("loop", parents=[path],
+                               description="Loop over .conf in a directory",
+                               help="Solve multiple configurations")
+    ploop = commands.add_parser("ploop", parents=[path],
+                                description="Loop over .conf in a directory [parallel version]",
+                                help="Solve multiple configurations")
 
     # show section subsubparsers : frame/probe/
     shw_cmds = shw.add_subparsers(dest='show_command',
@@ -105,7 +141,7 @@ def parse_args():
                         description="Display some simulation parameters",
                         help="display some simulation parameters")
 
-    # make section subsubparsers : movie/wav
+    # make section subsubparsers : movie/wav/template
     mak_cmds = mak.add_subparsers(dest='make_command',
                                   help='see -h for further help')
     mak_cmds.add_parser("movie", parents=[commons, view, data],
@@ -114,6 +150,9 @@ def parse_args():
     mak_cmds.add_parser("sound", parents=[commons, data],
                         description="Make sound files from existing results",
                         help="make sound files from existing results")
+    mak_cmds.add_parser("template", parents=[commons, data],
+                        description="Create basic configuration file",
+                        help="Create basic configuration file")
 
     return root.parse_args()
 
@@ -121,11 +160,8 @@ def parse_args():
 def solve(cfg, msh):
     """ Solve NS equations. """
 
-    # Simulation parameters
-    fld = Fields(msh, cfg)
-
     # Simulation
-    fdtd = FDTD(msh, fld, cfg)
+    fdtd = FDTD(msh, cfg)
     fdtd.run()
 
     if cfg.figures:
@@ -142,7 +178,6 @@ def solve(cfg, msh):
 def show(cfg, msh):
     """ Show simulation parameters and grid. """
 
-
     if cfg.args.show_command == 'parameters':
         headers.version()
         headers.check_geo(cfg)
@@ -165,7 +200,8 @@ def show(cfg, msh):
     elif cfg.args.show_command == 'frame':
         plt = graphics.Plot(cfg.datafile, quiet=cfg.quiet)
         plt.fields(view=cfg.args.view, iteration=cfg.args.nt, ref=cfg.args.ref,
-                   show_pml=cfg.show_pml, show_probes=cfg.show_probes, comp=cfg.args.comp)
+                   show_pml=cfg.show_pml, show_probes=cfg.show_probes,
+                   comp=cfg.args.comp)
 
     elif cfg.args.show_command == 'probes':
         plt = graphics.Plot(cfg.datafile, quiet=cfg.quiet)
@@ -175,7 +211,6 @@ def show(cfg, msh):
         plt = graphics.Plot(cfg.datafile, quiet=cfg.quiet)
         plt.spectrogram()
 
-
     else:
         headers.copyright()
         headers.version()
@@ -198,40 +233,91 @@ def make(cfg, _):
         _ = sounds.probes_to_wave(cfg.datafile)
 
 
-def main():
-    """ Main """
+def template(args):
+    """Make template."""
+    if not args.cfgfile:
+        print('Path/filename must be specified with -c option')
+    else:
+        cfgfile = pathlib.Path(args.cfgfile).expanduser()
+        path, filename = cfgfile.parent, cfgfile.stem + cfgfile.suffix
+        create_template(path=path, filename=filename)
+        print(f"{cfgfile} created")
 
-    # Parse arguments
-    args = parse_args()
+
+def loop(path):
+    """Loop simulations over .conf files"""
+    headers.copyright()
+    path = pathlib.Path(path)
+    files = [f for f in pathlib.os.listdir(path.expanduser())
+             if f.endswith('conf')]
+    ti = time.perf_counter()
+    for file in files:
+        print(f'Processing {file}...')
+        args = VirtualArguments(path.expanduser() / file)
+        run(args=args)
+        print(f'Making movie for {file}...')
+        cfg = CfgSetup(args=args)
+        plt = graphics.Plot(cfg.datafile, quiet=args.quiet)
+        plt.movie()
+    print(f'Simulations ended in {time.perf_counter() - ti:.2f}')
+
+
+def ploop(path):
+    """Loop simulations over .conf files [separate processes]."""
+    headers.copyright()
+    path = pathlib.Path(path)
+    files = [f for f in pathlib.os.listdir(path.expanduser())
+             if f.endswith('conf')]
+    ti = time.perf_counter()
+    with Pool(processes=(cpu_count() - 1)) as pool:
+        for file in files:
+            args = VirtualArguments(path.expanduser() / file)
+            pool.apply_async(_wrapper, args=(run, args))
+        pool.close()
+        pool.join()
+    print(f'Simulations ended in {time.perf_counter() - ti:.2f}')
+
+
+def run(args):
+    """Run nsfds2."""
 
     # Parse config file
     cfg = CfgSetup(args=args)
 
     # Mesh
-    if cfg.mesh == 'regular':
-        msh = mesh.Mesh((cfg.nx, cfg.nz), (cfg.dx, cfg.dz), origin=(cfg.ix0, cfg.iz0),
-                        bc=cfg.bc, obstacles=cfg.obstacles, Npml=cfg.Npml,
-                        stencil=cfg.stencil)
-    elif cfg.mesh == 'curvilinear':
-        fcurv = files.get_curvilinear(cfg)
-        msh = mesh.CurvilinearMesh((cfg.nx, cfg.nz), (cfg.dx, cfg.dz),
-                                   origin=(cfg.ix0, cfg.iz0),
-                                   bc=cfg.bc, obstacles=cfg.obstacles, Npml=cfg.Npml,
-                                   stencil=cfg.stencil, fcurvxz=fcurv)
-    elif cfg.mesh == 'adaptative':
-        msh = mesh.AdaptativeMesh((cfg.nx, cfg.nz), (cfg.dx, cfg.dz),
-                                  origin=(cfg.ix0, cfg.iz0),
-                                  bc=cfg.bc, obstacles=cfg.obstacles, Npml=cfg.Npml,
-                                  stencil=cfg.stencil)
+    msh = mesh.build(cfg.mesh, (cfg.nx, cfg.nz), (cfg.dx, cfg.dz),
+                     origin=(cfg.ix0, cfg.iz0),
+                     bc=cfg.bc, obstacles=cfg.obstacles,
+                     Npml=cfg.Npml, stencil=cfg.stencil,
+                     dilatation=cfg.Rx, Nd=cfg.Nd, only_pml=cfg.only_pml,
+                     fcurvxz=files.get_curvilinear(cfg))
 
     if args.command:
         globals()[args.command](cfg, msh)
     else:
         headers.copyright()
-        print('Must specify an action among solve/make/show')
+        print('Must specify an action among solve/make/show/loop')
         print('See nsfds2 -h for help')
 
 
+def main(args=None):
+    """ Main """
+
+    # Parse arguments
+    if not args:
+        args = parse_args()
+
+    # Command
+    if getattr(args, 'make_command', None) == 'template':
+        template(args)
+    elif getattr(args, 'command', None) == 'loop':
+        loop(args.path)
+    elif getattr(args, 'command', None) == 'ploop':
+        ploop(args.path)
+    else:
+        run(args)
+
+
 if __name__ == "__main__":
 
     os.nice(20)
